@@ -1,6 +1,6 @@
 /******************************************************************************/
 /* Skyutils functions - Chained list,socket,string,utils,web,threads, archive */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-03                             */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04                             */
 /******************************************************************************/
 
 /*
@@ -23,13 +23,14 @@
 #ifndef __SKY_UTILS_H__
 #define __SKY_UTILS_H__
 
-#define SKYUTILS_VERSION "2.6"
+#define SKYUTILS_VERSION "2.7"
+#define SKYUTILS_AUTHOR "Christophe Calmejane"
 
 #if defined(__MACH__) || defined(_AIX)
 #define __unix__
 #endif /* __MACH__ || _AIX */
 
-#ifdef __cplusplus
+#if defined(__cplusplus) && !defined(__BORLANDC__)
 extern "C" {
 #endif /* __cplusplus */
 
@@ -141,15 +142,15 @@ typedef struct SU_SList
 SU_PList SU_AddElementTail(SU_PList,void *);
 SU_PList SU_AddElementHead(SU_PList,void *);
 SU_PList SU_AddElementPos(SU_PList List,int Pos,void *Elem); /* First element is at pos 0 */
-SU_PList SU_DelElementElem(SU_PList,void *); /* THIS FONCTION DOESN'T FREE THE ELEMENT */
-SU_PList SU_DelElementTail(SU_PList); /* THIS FONCTION DOESN'T FREE THE ELEMENT */
-SU_PList SU_DelElementHead(SU_PList); /* THIS FONCTION DOESN'T FREE THE ELEMENT */
-SU_PList SU_DelElementPos(SU_PList,int); /* THIS FONCTION DOESN'T FREE THE ELEMENT */ /* First element is at pos 0 */
+SU_PList SU_DelElementElem(SU_PList,void *); /* This function does NOT free the element */
+SU_PList SU_DelElementTail(SU_PList); /* This function does NOT free the element */
+SU_PList SU_DelElementHead(SU_PList); /* This function does NOT free the element */
+SU_PList SU_DelElementPos(SU_PList,int); /* This function does NOT free the element */ /* First element is at pos 0 */
 void *SU_GetElementTail(SU_PList);
 void *SU_GetElementHead(SU_PList);
 void *SU_GetElementPos(SU_PList,int); /* First element is at pos 0 */
-void SU_FreeList(SU_PList); /* THIS FONCTION DOESN'T FREE THE ELEMENTS */
-void SU_FreeListElem(SU_PList); /* THIS FONCTION DOES FREE THE ELEMENTS */
+void SU_FreeList(SU_PList); /* This function does NOT free the elements */
+void SU_FreeListElem(SU_PList); /* This function DOES free the elements */
 unsigned int SU_ListCount(SU_PList);
 
 
@@ -174,7 +175,7 @@ typedef struct
 int SU_GetPortByName(char *port,char *proto); /* Returns port number from it's name */
 char *SU_GetMachineName(char *RemoteHost);    /* Extracts the machine name from a full host */
 char *SU_NameOfPort(char *Host);              /* Returns the host name matching the given ip */
-char *SU_AdrsOfPort(char *Host);              /* Returns the ip adrs mathing the given host */
+char *SU_AdrsOfPort(char *Host);              /* Returns the ip adrs matching the given host */
 
 SU_PServerInfo SU_CreateServer(int port,int type,bool ReUseAdrs);  /* Returns NULL on error */
 int SU_ServerListen(SU_PServerInfo SI);                            /* SOCKET_ERROR on Error */
@@ -216,6 +217,7 @@ char *SU_strcpy(char *dest,const char *src,int len); /* like strncpy, but doesn'
 char *SU_nocasestrstr(char *text, char *tofind);  /* like strstr(), but nocase */
 bool SU_strwcmp(const char *s,const char *wild); /* True if wild equals s (wild may use '*') */
 bool SU_nocasestrwcmp(const char *s,const char *wild); /* Same as strwcmp but without case */
+bool SU_strwparse(const char *s,const char *wild,char buf[],int size,char *buf_ptrs[],int *ptrs_count); /* True if wild equals s (wild may use '*') */
 unsigned int SU_strnlen(const char *s,unsigned int max); /* Returns MAX(length of a string,max) (not including terminating null char) */
 bool SU_ReadLine(FILE *fp,char S[],int len); /* Returns false on EOF */
 bool SU_ParseConfig(FILE *fp,char Name[],int nsize,char Value[],int vsize); /* Returns false on EOF */
@@ -230,6 +232,8 @@ unsigned char SU_tolower(unsigned char c);
 char *SU_strtoupper(char *s); /* s IS modified */
 char *SU_strtolower(char *s); /* s IS modified */
 bool SU_strcasecmp(const char *s,const char *p);
+char *SU_strerror(int ErrorCode); /* Returns a static string which describes the error code (errno on linux, GetLastError on windows) */
+
 #define SU_strdup(x) ((x)==NULL)?NULL:strdup(x)
 #ifdef _WIN32
 #define strcasecmp stricmp
@@ -325,12 +329,18 @@ unsigned long int SU_ElapsedTime(SU_TICKS t1,SU_TICKS t2,SU_CPUSPEED speed); /*
 #ifndef SU_MALLOC_ALIGN_SIZE
 #define SU_MALLOC_ALIGN_SIZE 16
 #else /* SU_MALLOC_ALIGN_SIZE */
-#if SU_MALLOC_ALIGN_SIZE < 2
-#error "SU_MALLOC_ALIGN_SIZE must be strictly greater than 2"
-#endif /* SU_MALLOC_ALIGN_SIZE < 2 */
+#if SU_MALLOC_ALIGN_SIZE < 8
+#error "SU_MALLOC_ALIGN_SIZE must be strictly greater than 8"
+#endif /* SU_MALLOC_ALIGN_SIZE < 8 */
 #endif /* !SU_MALLOC_ALIGN_SIZE */
+typedef void (SU_PRINT_FUNC)(bool Fatal,char *Txt, ...);
+void SU_SetPrintFunc(SU_PRINT_FUNC *Func);
+typedef void (SU_MEM_STATS_FUNC)(void *ptr,long int size,long int time,const char *file,int line);
+
 void *SU_malloc(long int size); /* Allocates a bloc of memory aligned on SU_MALLOC_ALIGN_SIZE */
 void *SU_calloc(long int nbelem,long int size); /* Allocates a bloc of memory aligned on SU_MALLOC_ALIGN_SIZE, and zeros it */
+void *SU_realloc(void *memblock,long int size); /* Reallocates a bloc of memory aligned on SU_MALLOC_ALIGN_SIZE */
+char *SU_strdup_memory(const char *in); /* Dups a string using a bloc of memory aligned on SU_MALLOC_ALIGN_SIZE */
 void SU_free(void *memblock);   /* Frees a bloc previously allocated using SU_malloc */
 #ifdef SU_MALLOC_TRACE
 #undef calloc
@@ -348,8 +358,10 @@ void *SU_calloc_trace(long int nbelem,long int size,char *file,int line);
 void *SU_realloc_trace(void *memblock,long int size,char *file,int line);
 char *SU_strdup_trace(const char *in,char *file,int line);
 void SU_free_trace(void *memblock,char *file,int line);
-void SU_alloc_trace_print(bool detail);
-void SU_check_memory(void);
+void SU_alloc_trace_print(bool detail); /* Print all allocated blocks (if detail is true), and total number of chunks */
+void SU_check_memory(void); /* Check memory heap */
+void SU_alloc_stats(SU_MEM_STATS_FUNC *Func); /* Call Func for each chunk of memory */
+unsigned long int SU_alloc_total_size(void); /* Returns the total size allocated */ /* Only available with SU_xx_trace functions */
 /* MALLOC_CHECK_ environment variable :
  *   0 : If a memory error occurs, execution silently goes on
  *   1 : If a memory error occurs, a debug message is printed in stdout, and execution goes on
@@ -479,6 +491,9 @@ SU_PForm SU_RetrieveForm(const char Ans[],const int num);
 
 char *SU_AddLocationToUrl(const char *URL,const char *Host,const char *Location,bool ssl_mode);
 
+/* Encodes an URL */
+char *SU_EncodeURL(const char URL_in[],char URL_out[],int URL_out_len);
+
 /* Skips white spaces before the string, then extracts it */
 char *SU_GetStringFromHtml(const char Ans[],const char TextBefore[]);
 
@@ -495,6 +510,7 @@ extern SU_PList SW_Cookies; /* SU_PCookie */
 #define SU_THREAD_ROUTINE_TYPE(x) void *(*x)(void *)
 #define SU_THREAD_ROUTINE(x,y) void * x(void *y)
 #define SU_END_THREAD(x) pthread_exit(x)
+#define SU_PROCESS_SELF getpid()
 #define SU_THREAD_SELF pthread_self()
 #define SU_THREAD_KEY_HANDLE pthread_key_t
 #define SU_THREAD_ONCE_HANDLE pthread_once_t
@@ -506,22 +522,37 @@ extern SU_PList SW_Cookies; /* SU_PCookie */
 #define SU_SEM_HANDLE sem_t
 #define SU_SEM_WAIT(x) sem_wait(&(x))
 #define SU_SEM_POST(x) sem_post(&(x))
+#define SU_MUTEX_HANDLE pthread_mutex_t
+#define SU_MUTEX_WAIT(x) pthread_mutex_lock(&(x))
+#define SU_MUTEX_POST(x) pthread_mutex_unlock(&(x))
+#define SU_CRITICAL pthread_mutex_t
+#define SU_CRITICAL_ENTER(x) pthread_mutex_lock(&(x))
+#define SU_CRITICAL_LEAVE(x) pthread_mutex_unlock(&(x))
+#define SU_CRITICAL_TRY_AND_ENTER(x) (pthread_mutex_trylock(&(x)) == 0)
 #else /* !__unix__ */
 #define SU_THREAD_HANDLE unsigned long
 #define SU_THREAD_ROUTINE_TYPE(x) void (__cdecl *x)(void *)
 #define SU_THREAD_ROUTINE(x,y) void __cdecl x(void *y)
 #define SU_END_THREAD(x) _endthread()
+#define SU_PROCESS_SELF GetCurrentProcessId()
 #define SU_THREAD_SELF GetCurrentThreadId()
 #define SU_THREAD_KEY_HANDLE DWORD
 #define SU_THREAD_ONCE_HANDLE DWORD
 #define SU_THREAD_ONCE_INIT 0
 #define SU_THREAD_ONCE(x,y) y()
 #define SU_THREAD_GET_SPECIFIC(x) TlsGetValue(x)
-#define SU_THREAD_SET_SPECIFIC(x,y) TlsSetValue(x,y)
+#define SU_THREAD_SET_SPECIFIC(x,y) TlsSetValue(x,(LPVOID)y)
 #define SU_THREAD_DESTROY_SPECIFIC(x,y) x(y)
 #define SU_SEM_HANDLE HANDLE
 #define SU_SEM_WAIT(x) WaitForSingleObject(x,INFINITE)
 #define SU_SEM_POST(x) ReleaseSemaphore(x,1,NULL)
+#define SU_MUTEX_HANDLE HANDLE
+#define SU_MUTEX_WAIT(x) WaitForSingleObject(x,INFINITE)
+#define SU_MUTEX_POST(x) ReleaseMutex(x)
+#define SU_CRITICAL CRITICAL_SECTION
+#define SU_CRITICAL_ENTER(x) EnterCriticalSection(&(x))
+#define SU_CRITICAL_LEAVE(x) LeaveCriticalSection(&(x))
+#define SU_CRITICAL_TRY_AND_ENTER(x) TryEnterCriticalSection(&(x))
 #endif /* __unix__ */
 /* Create a new thread */
 bool SU_CreateThread(SU_THREAD_HANDLE *Handle,SU_THREAD_ROUTINE_TYPE(Entry),void *User,bool Detached); /* True on success */
@@ -541,6 +572,17 @@ bool SU_FreeSem(SU_SEM_HANDLE *Handle);
 bool SU_CreateThreadKey(SU_THREAD_KEY_HANDLE *Handle,SU_THREAD_ONCE_HANDLE *Once,void (*destroyts)(void *)); /* True on success */
 /* Block all signals for the calling thread */
 void SU_ThreadBlockSigs(void);
+/* Create a new mutex */
+bool SU_CreateMutex(SU_MUTEX_HANDLE *Handle,const char MutexName[]); /* True on success */
+/* Free a mutex */
+bool SU_FreeMutex(SU_MUTEX_HANDLE *Handle);
+/* Create a critical section */
+#define SU_CRITICAL_TYPE_ANY           1
+#define SU_CRITICAL_TYPE_RECURSIVE     2
+#define SU_CRITICAL_TYPE_NON_RECURSIVE 3
+bool SU_CriticalInit(SU_CRITICAL *Crit,int Type); /* True on success */
+/* Free a critical section */
+bool SU_CriticalDelete(SU_CRITICAL *Crit);
 #endif /* !SU_INCLUDE_NO_THREAD */
 
 
@@ -571,9 +613,9 @@ void *SU_DL_GetSym(SU_DL_HANDLE handle, const char Name[]);
 /* **************************************** */
 #ifdef SU_USE_ARCH
 
-#define SU_ARCH_COMP_NONE 0
-#define SU_ARCH_COMP_Z    8
-#define SU_ARCH_COMP_BZ   16
+#define SU_ARCH_COMP_NONE 1
+#define SU_ARCH_COMP_Z    2
+#define SU_ARCH_COMP_BZ   4
 
 typedef struct
 {
@@ -606,11 +648,11 @@ typedef struct
 typedef unsigned int SU_AR_COMP_TYPE;
 
 /* *** Reading functions *** */
-/* Opens a skyutils archive file */
+/* Opens a skyutils archive file (or a binary file [exe/dll] if the archive is selfcontained) */
 SU_PArch SU_AR_OpenArchive(const char FileName[]);
 /* Reads resource ResNum (0 is the first one) (NULL if failed) */
 SU_PRes SU_AR_ReadRes(SU_PArch Arch,const unsigned int ResNum,bool GetData);
-/* Save resource ResNum to FileName (0 is the first one) (NULL if failed) */
+/* Save resource ResNum to FileName (0 is the first one) (true on success) */
 bool SU_AR_ReadResFile(SU_PArch Arch,const unsigned int ResNum,const char FileName[]);
 
 
@@ -643,25 +685,36 @@ SU_AR_COMP_TYPE SU_AR_SupportedComps(void);
 #else /* !_WIN32 */
 #define HKEY void *
 #endif /* _WIN32 */
-#define SU_RB_ERR_SUCCESS 0
+
+#define SU_RB_ERR_SUCCESS       0
 #define SU_RB_ERR_ACCESS_DENIED 1
-#define SU_RB_ERR_WRONG_TYPE 2
-#define SU_RB_ERR_INVALID_KEY 3
+#define SU_RB_ERR_WRONG_TYPE    2
+#define SU_RB_ERR_INVALID_KEY   3
+#define SU_RB_ERR_INVALID_PATH  4
+#define SU_RB_ERR_LOCK_FAILED   5
+#define SU_RB_ERR_PREMATURE_EOF 6
+#define SU_RB_ERR_WRITE_ERROR   7
+#define SU_RB_ERR_INVALID_TYPE  8
+#define SU_RB_ERR_CORRUPTED     9
 
 int SU_RB_GetLastError();
-void SU_RB_FlushRegistry(void);
 
+/* 'Key' parameter must be a succession of subkeys separated by '\', and finished by a keyname. To set/get the default value of a subkey, put a trailing '\' on 'Key' */
 void SU_RB_GetStrValue(const char Key[],char *buf,int buf_len,const char Default[]);
 int SU_RB_GetIntValue(const char Key[],int Default);
 bool SU_RB_SetStrValue(const char Key[],const char Value[]);
 bool SU_RB_SetIntValue(const char Key[],int Value);
-bool SU_RB_DelKey(const char Key[]);
+bool SU_RB_DelKey(const char Key[]); /* Recursively deletes All sub-keys and values of Key (and Key itself) */
 bool SU_RB_DelValue(const char Key[]);
 
+bool SU_RB_OpenRegistry(const char RegistryPath[]); /* Opens registry for reading/writing. True on success - Only one registry opened at one time, close previous one if opened */
+bool SU_RB_CloseRegistry(void); /* Flush all writings since registry was opened and close it. True on success. If failed, nothing was flushed */
+
 HKEY SU_RB_OpenKeys(const char Key[],int Access); /* Opened HKEY or 0 on error */
 HKEY SU_RB_CreateKeys(const char Key[]); /* Created HKEY or 0 on error */
-bool SU_RB_EnumStrValue(HKEY Key,int Idx,char *Name,int name_len,char *Value,int value_len); /* True on Success. False when no more values available */
-bool SU_RB_EnumIntValue(HKEY Key,int Idx,char *Name,int name_len,int *Value); /* True on Success. False when no more values available */
+bool SU_RB_EnumKey(HKEY Key,int Idx,char *Name,int name_len); /* True on Success. False when no more values available */
+bool SU_RB_EnumStrValue(HKEY Key,int Idx,char *Name,int name_len,char *Value,int value_len); /* True on Success. False when no more values available */ /* All values must be of same type in the HKEY (int or string) */
+bool SU_RB_EnumIntValue(HKEY Key,int Idx,char *Name,int name_len,int *Value); /* True on Success. False when no more values available */ /* All values must be of same type in the HKEY (int or string) */
 void SU_RB_CloseKey(HKEY Key);
 #endif /* !SU_INCLUDE_NO_REG */
 
@@ -680,8 +733,8 @@ void SU_RB_CloseKey(HKEY Key);
 /* ** Global options */
 void SU_DBG_SetFlags(const SU_u64 Flags);
 void SU_DBG_SetOutput(const SU_u16 Output);
-/* If PrintTime is set to true, date/time is also printed. CurrentThreadId is printed if PrintThreadId is set to true */
-void SU_DBG_SetOptions(const bool PrintTime,const bool PrintThreadId);
+/* If PrintTime is set to true, date/time is also printed. CurrentProcessId/CurrentThreadId is printed if PrintProcessId/PrintThreadId is set to true */
+void SU_DBG_SetOptions(const bool PrintTime,const bool PrintProcessId,const bool PrintThreadId);
 
 /* ** Printf Output Options ** */
 /* Ansi color is used if AnsiColor is set to true */
@@ -700,27 +753,35 @@ void SU_DBG_OUT_SOCKET_SetOptions(const char HostName[],const int Port);
 void SU_DBG_PrintDebug(const SU_u64 Type,char *Txt, ...);
 
 /* **** Debug Env vars **** */
-/*
-   Global env var : SU_DBG_HELP = Print this help
-                    SU_DBG_OUTPUT = {printf,console,file,socket,popup} (space separated)
-                    SU_DBG_FLAGS = <Flags> (Flags is a 64bits bitfield defining which flags to output)
-                    SU_DBG_OPTIONS = {time,thread} (space separated)
-    printf env var : SU_DBG_OUT_PRINTF = {0|1} (AnsiColor boolean)
-    console env var : SU_DBG_OUT_CONSOLE = <WindowName>
-    file env var : SU_DBG_OUT_FILE = {0|1} <FileName> (0|1 is DeletePreviousLog boolean)
-    socket env var : SU_DBG_OUT_SOCKET = <HostName:Port>[ <HostName:Port>] ...
-    popup env var : N/A
-*/
+#define SU_DBG_HELP_MESSAGE "SkyUtils Debug : Environment variables HELP (overrides application init on the first 'SU_DBG_PrintDebug' call) :\n\n" \
+"   Global env var : SU_DBG_HELP = Print this help\n" \
+"                    SU_DBG_OUTPUT = {printf,console,file,socket,popup} (space separated)\n" \
+"                    SU_DBG_FLAGS = <Flags> (Flags is a 64bits bitfield defining which flags to output)\n" \
+"                    SU_DBG_OPTIONS = {time,process,thread} (space separated)\n" \
+"    printf env var : SU_DBG_OUT_PRINTF = {0|1} (AnsiColor boolean)\n" \
+"    console env var : SU_DBG_OUT_CONSOLE = <WindowName>\n" \
+"    file env var : SU_DBG_OUT_FILE = {0|1} <FileName> (0|1 is DeletePreviousLog boolean)\n" \
+"    socket env var : SU_DBG_OUT_SOCKET = <HostName:Port>[ <HostName:Port>] ...\n" \
+"    popup env var : N/A\n" \
+"\n"
+
+#ifdef _WIN32
+/* Special Windows Config Function (pops up a GUI to choose debug options, overriding application init, and env vars) - Returns true if debug options were changed */
+/* hInstance must be the Instance of the module who is linked with the skyutils library (use GetModuleHandle(NULL) to retrieve calling process' HINSTANCE) */
+bool SU_DBG_ChooseDebugOptions_Windows(HINSTANCE hInstance,HWND Parent);
+#endif /* _WIN32 */
 
 
 
+/* **************************************** */
+/*              MISC   functions            */
+/* **************************************** */
+
 /* Dummy functions used by configure, to check correct version of skyutils */
 /* Remove old ones if compatibility has been broken */
-void SU_Dummy204(void);
-void SU_Dummy205(void);
-void SU_Dummy206(void);
+void SU_Dummy207(void);
 
-#ifdef __cplusplus
+#if defined(__cplusplus) && !defined(__BORLANDC__)
 }
 #endif /* __cplusplus */
 
