@@ -1,6 +1,6 @@
 /****************************************************************/
 /* Web unit                                                     */
-/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-03               */
+/* (c) Christophe CALMEJANE (Ze KiLleR) - 1999-04               */
 /****************************************************************/
 
 /*
@@ -1207,7 +1207,7 @@ SU_PAnswer WaitForAnswer(SU_SOCKET Sock,SU_PHTTPActions Act,bool proxy,SSL *ssl)
     if(SU_DebugLevel >= 5)
       DumpPage(NULL,Ans->Data,Ans->Data_Length);
 #endif /* __unix__ */
-    if((Act->FileName != NULL) && ((Act->Command == ACT_GET) || (Act->Command == ACT_POST)))
+    if((Ans->Data != NULL) && (Act->FileName != NULL) && ((Act->Command == ACT_GET) || (Act->Command == ACT_POST)))
       DumpPage(Act->FileName,Ans->Data,Ans->Data_Length);
   }
   return Ans;
@@ -1278,10 +1278,42 @@ bool SU_SendProxySSLConnect(SU_SOCKET Sock,char *Host,int Port,int *Code)
   return found;
 }
 
+char *SU_EncodeURL(const char URL_in[],char URL_out[],int URL_out_len)
+{
+  char NB[10];
+  int i,pos,val;
+
+  pos = 0;
+  for(i=0;i<strlen(URL_in);i++)
+  {
+    if((URL_in[i] >='!') && (URL_in[i] <='~'))
+    {
+      if((pos+1) >= URL_out_len)
+      {
+        break;
+      }
+      URL_out[pos++] = URL_in[i];
+    }
+    else
+    {
+      if((pos+3) >= URL_out_len)
+      {
+        break;
+      }
+      URL_out[pos++] = '%';
+      snprintf(NB,sizeof(NB),"%.2x",URL_in[i]);
+      URL_out[pos++] = NB[strlen(NB)-2];
+      URL_out[pos++] = NB[strlen(NB)-1];
+    }
+  }
+  URL_out[pos] = 0;
+  return URL_out;
+}
+
 int SU_ExecuteActions(SU_PList Actions)
 {
   SU_PList Ptr = Actions;
-  SU_PList ActRec;
+  SU_PList ActRec = NULL;
   SU_PAnswer Ans;
   SU_THTTPActions Act;
   char URL_OUT[URL_BUF_SIZE];
@@ -1410,6 +1442,8 @@ int SU_ExecuteActions(SU_PList Actions)
               if(((SU_PHTTPActions)Ptr->Data)->CB.OnMoved != NULL)
                 ((SU_PHTTPActions)Ptr->Data)->CB.OnMoved(Ans,((SU_PHTTPActions)Ptr->Data)->User);
               memset(&Act,0,sizeof(Act));
+              if(((SU_PHTTPActions)Ptr->Data)->FileName)
+                Act.FileName = strdup(((SU_PHTTPActions)Ptr->Data)->FileName);
               Act.User = ((SU_PHTTPActions)Ptr->Data)->User;
               memcpy(&Act.CB,&((SU_PHTTPActions)Ptr->Data)->CB,sizeof(Act.CB));
               Act.Command = ACT_GET;
@@ -1426,7 +1460,7 @@ int SU_ExecuteActions(SU_PList Actions)
               Code = GetHostFromURL(Ans->Location,((SU_PHTTPActions)Ptr->Data)->Host,sizeof(Act.Host),true,Act.URL,&PortConnect,"",&ssl_mode);
               if(Code != 0)
                 return Code;
-              SU_strcpy(Act.URL,Ans->Location,sizeof(Act.URL));
+              SU_EncodeURL(Ans->Location,Act.URL,sizeof(Act.URL));
               Act.URL_Params = NULL;
               if(((SU_PHTTPActions)Ptr->Data)->Referer != NULL)
                 Act.Referer = ((SU_PHTTPActions)Ptr->Data)->Referer;
@@ -1434,6 +1468,9 @@ int SU_ExecuteActions(SU_PList Actions)
                 Act.Referer = ((SU_PHTTPActions)Ptr->Data)->URL;
               ActRec = SU_AddElementHead(NULL,&Act);
               SU_ExecuteActions(ActRec);
+              if(Act.FileName)
+                free(Act.FileName);
+              ActRec = SU_DelElementHead(ActRec);
               break;
             case 403 : /* Forbidden */
               if(((SU_PHTTPActions)Ptr->Data)->CB.OnForbidden != NULL)
@@ -2222,7 +2259,7 @@ void SU_FreeForm(SU_PForm Form)
 char *SU_AddLocationToUrl(const char *URL,const char *Host,const char *Location,bool ssl_mode)
 {
   char *ptr = NULL;
-  int len,i;
+  int len,i,pos = 0;
 
   if(strncasecmp(Location,"http://",7) != 0) /* Relative path */
   {
@@ -2243,6 +2280,7 @@ char *SU_AddLocationToUrl(const char *URL,const char *Host,const char *Location,
         SU_strcpy(ptr,URL,len);
       else
         snprintf(ptr,len,"http%s://%s%s",ssl_mode?"s":"",Host,URL);
+
       if(strcmp(ptr+strlen("http://")+(ssl_mode?1:0),Host) == 0) /* If requested the root of the site */
         SU_strcat(ptr,"/",len);
       else
@@ -2258,8 +2296,24 @@ char *SU_AddLocationToUrl(const char *URL,const char *Host,const char *Location,
           i--;
         }
       }
+      /* Here, ptr have a trailing '/' */
+      /* Check for '../' in Location */
+      while(strncmp(Location+pos,"../",3) == 0)
+      {
+        i = strlen(ptr) - 1 - 1; /* Start from before the trailing '/' */
+        while(i>=0)
+        {
+          if(ptr[i] == '/')
+          {
+            ptr[i+1] = 0; /* Go back a directory level */
+            break;
+          }
+          i--;
+        }
+        pos += 3;
+      }
     }
-    SU_strcat(ptr,Location,len);
+    SU_strcat(ptr,Location+pos,len);
   }
   else
     ptr = strdup(Location);
